:userdoc.

:title.Extended Spreadsheet


.*********************************************************
:h1 res=99. Disclaimer
:i1. Disclaimer
:p.THIS SOFTWARE IS PROVIDED BY THOMAS KORFHAGE "AS IS'' AND ANY EXPRESSED
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THOMAS KORFHAGE BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

.*********************************************************
:h1 res=100 . Extension of VisPro Rexx Spreadsheet Object
:i1. General

:p.The VisPro Rexx Spreadsheet Object is limited in the way you can start editing
a field. You have to ALT+LeftClick on the field to get it into the edit state.

:p.I didn't like that to be the only way, so i searched for a way to enhance the spreadsheet control.
:p.I did this first for the VisProC object. I subclassified the VPSpreadsheetClass.
First i thought i had to create a complete new object with the VPTOOL kit to do the same for Rexx.
Then i got the idea to try the subclassing in an external rexx-dll. I found that i seems so work.
:p.But i could not use class subclassing, because there is no way in VisProRexx to make the call at the 
wright time (between registration of the standard VisProSpreadSheetObject and the
first creation of an instance of it, if the spreadsheet is on the first (Main) window form).
:p.So i decided to use window subclassing. That's what is done if you call SbcSpsht(window, id).
:p.The Spreadsheet Object (Control id on form window) will be subclassed. You can then set the desired edit style and/or let the spreadsheet perform a display optimization.

:p.This sample was build with screen resolution 1280x1024. May be it looks bad on VGA.

:p.:link reftype=hd res=999.Thomas Korfhage:elink. Sep. 1997
:warning.
This is the first time i tried the window subclassing in an external rexx-dll. During my test cycle (German Warp4 no FP)  i could not watch any side effects and problems. This does not mean, that i would not expect them to arise in the future.
So read the :link reftype=hd res=99.Disclaimer:elink.!:ewarning.

.*********************************************************
:h1 res=110. How to enable the extensions
:i1. Enable the extensions
:p.To get access to the extensions, you first have to register the rexx function SbcSpsht
:xmp.
/* Event Form events, Opened */
/* Register the extension and subclass the spreadsheet */
call RxFuncAdd 'SbcSpsht', 'SBCSPSHT', 'SbcSpsht'
SbcSpshtRV = SbcSpsht(window, 1000);
:exmp.
:p.SbcSpsht must be called for every spreadsheet object after its creation. Successfully subclassing is shown by a return value of "1", "0" means subclassing could not be done.

.*********************************************************
:h1 res=120. How to configure the edit style
:i1. Configure the edit style
:p.To set the desired edit style, a message has to be send to the control. Use the Create link "Send message 2 longs" from the spreadsheet control.
:xmp.
/* Set the edit style */
Style = :link reftype=hd res=112.SPRS.ENTER_EDITS:elink. + :link reftype=hd res=112.SPRS.CHAR_EDITS:elink.
value=VpItem(window,1000,'SENDMSG', :link reftype=hd res=112.SPRM.SET_EDTSTYLE:elink., Style, :link reftype=hd res=113.EditKey:elink.);
:exmp.
:p.The edit styles are non exclusive flags, so they can be added.

.*********************************************************
:h1 res=130. Optimize the Display
:i1. Optimize the Display
:p.The extension also includes the optimizing of the Spreadsheet widths  and heights according to font in use.
:p.To optimize the spreadsheet, a message has to be send to the control. Use the Create link "Send message 4 shorts" from the spreadsheet control.
:parml compact break=none.
:pt.Short 1
:pd.Column to optimize, -1 for all, 0 for the row label column
:pt.Short 2
:pd.Column and/or Header, 1 for column width optimization, 2 for header height optimization, 3 for both
:pt.Short 3
:pd.Additional horizontal space 
:pt.Short 4
:pd.Additional vertical space
(space for rows) * 256 + space for header
:eparml.
:xmp.
/* Set the edit style */
value=VpItem(window,1000,'SENDMSG', :link reftype=hd res=112.SPRM.OPTIMZE:elink., :link reftype=hd res=112.SPRS.OPTIMZE_ALLCOLS:elink., :link reftype=hd res=112.SPRS.OPTIMZE_BOTH:elink., '10', '10');
:exmp.
:p.Be sure to set the max number of cells before this call, because the text of up to all fields must be checked to find the longest.
:p.If You have set the max number of cells and performed the optimization, You my size the spreadsheet to the size that displays all fields. Be sure the control will fit into your window!
:xmp.
/* Get item position 1000 Spreadsheet */
xy=VpItem(window,1000,'ITEMPOS')
PARSE VAR xy x y
/* Set item position/size 1000 Spreadsheet */
CALL VpItem window,1000,'SETITEMPOS',x,y,20000,20000
/* Repaint item 1000 Spreadsheet */
rc=VpItem(window,1000,'PAINT')
:exmp.

:p.This uses the functionality build into the original Vispro Spreadsheet to reduce it's size to the maximal needed area.


.*********************************************************
:h1 res=140. Readonly Fields/Columns/Rows
:i1. Readonly 
:p.Any Field can be set readonly.
:p.To set fields readonly, a message has to be send to the control. Use the Create link "Send message 4 shorts" from the spreadsheet control.
:parml compact break=none.
:pt.Short 1
:pd.Column 1-n, -1 for all columns of a row
:pt.Short 2
:pd.Row 1-n, -1 for all of rows of a column
:pt.Short 3
:pd.Readonly Flag, 1 set Readonly, 0 reset Readonly
:pt.Short 4
:pd.Not used, should be 0
:eparml.
:p.Set field 1,2 readonly.
:xmp.
value=VpItem(window,1000,'SENDMSG', SPRM.SET_READONLY, 1, 2, 1, 0);
:exmp.
:p.Set row 2 readonly.
:xmp.
value=VpItem(window,1000,'SENDMSG', SPRM.SET_READONLY, -1, 2, 1, 0);
:exmp.
:p.Set coulumn 3 readonly.
:xmp.
value=VpItem(window,1000,'SENDMSG', SPRM.SET_READONLY, 3, -1, 1, 0);
:exmp.
:p.Set every field readonly.
:xmp.
value=VpItem(window,1000,'SENDMSG', SPRM.SET_READONLY, -1, -1, 1, 0);
:exmp.
:p.Reset readonly for every field.
:xmp.
value=VpItem(window,1000,'SENDMSG', SPRM.SET_READONLY, -1, -1, 0, 0);
:exmp.
:p.This functionality is limited to 1000 rows and/or 1000 columns!

.*********************************************************
:h1 res=150. Change color for cell in edit mode
:i1. Color in edit mode
:p.The extension also includes the posibility to change the color for a cell that is in edit mode.
:p.To optimize the spreadsheet, a message has to be send to the control. Use the Create link "Send message 4 longs" from the spreadsheet control.
:parml compact break=none.
:pt.Long 1
:pd.Fore ground color, use a RGB - value
:pt.Long 2
:pd.Back ground color, use a RGB - value
:eparml.
:xmp.
/* Set the edit Color to black on red */
Bcolor = ((255 * 256) + 0)*256+0
Fcolor = 0
value=VpItem(window,1000,'SENDMSG', :link reftype=hd res=112.SPRM.SET_EF_COLORS:elink., Fcolor, Bcolor);
:exmp.

.*********************************************************
:h1 res=160. Enter edit mode for a cell
:i1. Enter edit mode
:p.The extension also includes the posibility directly start the edit mode for one cell.
:p.To optimize the spreadsheet, a message has to be send to the control. Use the Create link "Send message 4 longs" from the spreadsheet control.
:parml compact break=none.
:pt.Long 1
:pd.Column of Cell
:pt.Long 2
:pd.Row of CellEnter edit mode
:eparml.
:xmp.
/* Start edit mode for cell 3,3 */
value=VpItem(window,1000,'SENDMSG', :link reftype=hd res=112.SPRM.EDIT_CELL:elink., 3, 3);
:exmp.

.*********************************************************
:h1 res=112. Constant definitions
:i1. Constant definitions
:p.To make code more readable following definitions are used:
:xmp.
SPRM.SET_EDTSTYLE = '0x101b'
SPRM.GET_EDTSTYLE = '0x101c'
SPRS.ENTER_EDITS  = 256
SPRS.CHAR_EDITS   = 512
SPRS.EDITFULLSEL  = 1024
SPRS.EDITFIXLEN   = 2048
SPRS.ALLOWDELETE  = 4096
SPRS.GO_LEFT      = 8192
SPRS.GO_DOWN      = 16384

SPRS.VIRTUALKEY   = 65536
SPRS.KEY_SHIFT    = 16777216
SPRS.KEY_CTRL     = 33554432
SPRS.KEY_ALT      = 67108864

SPRM.OPTIMZE      = '0x101a'
SPRS.OPTIMZE_ALLCOLS   =-1
SPRS.OPTIMZE_COLS      =1
SPRS.OPTIMZE_HEADER    =2
SPRS.OPTIMZE_ROW_HEIGHT =4

SPRM.SET_READONLY  = '0x101e'
SPRM.SET_EF_COLORS = '0x101f'
SPRM.EDIT_CELL     = '0x1020'
:exmp.

.*********************************************************
:h1 res=113. Additional Edit Key
:i1. Edit Key
:p.Beneath the Enterkey one other user defined key can be set to act in the same way.
:p.The Key must be given as a special constant when setting the edit style.

:xmp.
value = SPRS.VIRTUALKEY
if shft then value = value + SPRS.KEY_SHIFT;
if ctrl then value = value + SPRS.KEY_CTRL;
if alt then value = value + SPRS.KEY_ALT;
value = value + 33;		/* key 33 is F2 */
value=VpItem(window,1000,'SENDMSG', SPRM.SET_EDTSTYLE, Style, value);
:exmp.

.*********************************************************
:h1 res=333. RXWAITUNTIL
:i1. Funtion RXWAITUNTIL

:p.Included within the spcspsht.dll is the function RXWAITUNTIL.
:p.It can be used, to suspend a the REXX programm until a specified time is reached.
:p.ActualDateString = RXWAITUNTIL(h,m,s)
:parml.
:pt.h
:pd.hour to wait for (0-23). If h is greater 23 it matches any hour.
:pt.m
:pd.minute to wait for (0-23). If m is greater 59 it matches any minute.
:pt.s
:pd.second to wait for.
:eparml.
:p.If RXWAITUNTIL is called w/o any parameter it will be the same than RXWAITUNTIL(99,99,0). It waits until the next minute start.
.*********************************************************
:h1 res=999. Contact Thomas Korfhage
:i1. Contact Thomas Korfhage

:p.100271.16@compuserve.com
:p.  or
:p.korfhage@t-online.de

:euserdoc.










